#!/usr/bin/env node

///
/// $Release: 0.1.1 $
/// $Copyright: copyright(c) 2011 kuwata-lab.com all rights reserved $
/// $License: MIT License $
///

"use strict";

var util = require("util");
var picotest = exports;
picotest.__version__ = "$Release: 0.1.1 $".split(/ /)[1];

picotest.run = function run(test_obj, expected_count) {
  var count = 0, stack = [], errors = [];
  function run_func(func, before, after, this_) {
    count++;
    if (this_ === undefined) this_ = {};
    if (before) { before.apply(this_); }
    try { func.apply(this_); util.print("."); }
    catch (ex) { util.print("E"); errors.push([ex, stack.join(" > ")]); }
    finally { if (after) { after.apply(this_); } }
  }
  function run_obj(obj) {
    var before = obj.BEFORE, after = obj.AFTER, this_ = obj.THIS;
    if (typeof(this_) === "function") this_ = this_();
    for (var k in obj) {
      if (k === "BEFORE" || k === "AFTER" || k === "THIS") continue;
      stack.push(k);
      var v = obj[k];
      if      (typeof(v) === 'function') { run_func(v, before, after, this_); }
      else if (typeof(v) === 'object')   { run_obj(v); }
      else { throw new Error("picotest.run(): unexpected value: " + util.inspect(v)); }
      stack.pop(k);
    }
  };
  function print_error(err, location, sep) {
    util.print(sep + "\n");
    util.print("[Error] " + location + "\n");
    util.print(err.stack.replace(/\n    at run_func \(.*\bpicotest.js:(.|\n)*/, "\n"));
  }
  function print_footer(n_passed, n_errors, sep) {
    if (n_errors > 0) { util.print(sep + "\n"); }
    util.print("## passed:" + n_passed + ", error:" + n_errors + "\n");
  }
  //
  var sep = "----------------------------------------------------------------------";
  var n_tests = count_tests(test_obj, 0);
  run_obj(test_obj);
  util.print("\n");
  for (var i = 0, n = errors.length; i < n; i++) {
    var pair = errors[i];
    print_error(pair[0], pair[1], sep);
  }
  var n_errors = errors.length;
  print_footer(count - n_errors, n_errors, sep);
  if (count !== n_tests) {
    util.print("## planned " + n_tests + " tests but " + count + " tests done.\n");
  }
  return n_errors;
};

picotest.tap = function tap(test_obj) {
  function print_ok(desc) {
    util.print("ok " + count + " - " + desc + "\n");
  }
  function print_not_ok(desc, err) {
    util.print("not ok " + count + " - " + desc + "\n");
    var str = err.stack.replace(/\n    at run_func \(.*\bpicotest.js:(.|\n)*/, '');
    util.print(str.replace(/^/gm, "# "));
    util.print("\n");
  }
  var count = 0, stack = [], n_errors = 0;
  function run_func(desc, func, before, after, this_) {
    count++;
    if (this_ === undefined) this_ = {};
    if (before) { before.apply(this_); }
    try { func.apply(this_); print_ok(desc); }
    catch (ex) { n_errors++; print_not_ok(desc, ex); }
    finally { if (after) { after.apply(this_); } }
  }
  function run_obj(desc, obj) {
    stack.push(desc);
    util.print("### " + stack.join(" > ") + "\n");
    run_tests(obj);
    stack.pop();
  }
  function run_tests(obj) {
    var before = obj.BEFORE, after = obj.AFTER, this_ = obj.THIS;
    if (typeof(this_) === "function") this_ = this_();
    for (var k in obj) {
      if (k === "BEFORE" || k === "AFTER" || k === "THIS") continue;
      var v = obj[k];
      if      (typeof(v) === 'function') { run_func(k, v, before, after, this_); }
      else if (typeof(v) === 'object')   { run_obj(k, v); }
      else { throw new Error("function or object expected but got " + util.inspect(v)); }
    }
  };
  //
  var n_tests = count_tests(test_obj, 0);
  util.print("1.." + n_tests + "\n");
  run_tests(test_obj);
  if (count !== n_tests) {
    util.print("# Looks like you planned " + n_tests + " tests but ran " + count + ".\n");
  }
  return n_errors;
};

function count_tests(obj, n) {
  if (n == null) { n = 0; }
  for (var k in obj) {
    if (k === "BEFORE" || k === "AFTER" || k === "THIS") continue;
    var v = obj[k];
    if      (typeof(v) === "function") { n++; }
    else if (typeof(v) === "object") { n = count_tests(v, n); }
    else { throw new Error("function or object expected but got " + util.inspect(v)); }
  }
  return n;
}
picotest.count_tests = count_tests;

picotest.withDummyPrint = function withDummyPrint(block) {
  var buf = {output: ""};
  var bkup = util.print;
  util.print = function(args) {
    for (var i = 0, n = arguments.length; i < n; i++) {
      buf.output += arguments[i];
    }
  };
  try { block(buf); }
  finally { util.print = bkup; }
  return buf.output;
};

var help = (
  ''
    + 'Usage: picotest.js [options] file_or_directory ...\n'
    + '  -h, --help  : show help\n'
    + '  -v          : print version\n'
    + '  -st         : TAP style\n'
    + '  -sp         : xUnit style (default)\n'
);

picotest.main = function main(args) {
  var fs   = require("fs"),
      path = require("path");
  var runner = picotest.run, n_errors = 0;
  if (args == null) args = process.argv.slice(1);
  if (args.length > 0) {
    var fname = args[0];
    try { fname = fs.realpathSync(fname); }
    catch (ignore) { }
    if (fname === __filename) args = args.slice(1);
  }
  function run_path(fpath, rexp) {
    var stat = fs.statSync(fpath);
    if (stat.isFile()) {
      if (! rexp || path.basename(fpath).match(rexp)) {
        var test_obj = require(fs.realpathSync(fpath));
        n_errors += runner(test_obj);
      }
    }
    else if (stat.isDirectory()) {
      var children = fs.readdirSync(fpath);
      for (var i = 0, n = children.length; i < n; i++) {
        var child = children[i];
        if (child === "." || child === "..") continue;
        run_path(path.join(fpath, child), rexp);
      }
    }
  }
  //
  //var filter_rexp = /^(test_.*|.*_test)\.js$/;
  var filter_rexp = null;
  for (var i = 0, n = args.length; i < n; i++) {
    if (args[i] === "-h" || args[i] === "--help") { util.print(help); break; }
    else if (args[i] === "-v") { util.print(picotest.__version__ + "\n"); break; }
    else if (args[i] === "-st") { runner = picotest.tap; }
    else if (args[i] === "-sp") { runner = picotest.run; }
    else { run_path(args[i], filter_rexp); }
  }
  return n_errors;
};

if (require.main === module) process.exit(picotest.main());
