///
/// $Release: 0.1.1 $
/// $Copyright: copyright(c) 2011 kuwata-lab.com all rights reserved $
/// $License: MIT License $
///

var assert = require("assert"),
    util   = require("util");

var picotest = require("picotest");

function should(desc, block) {
  var status = "ERROR";
  try {
    block();
    status = "ok";
  }
  finally {
    console.log("[" + status + "] " + desc);
  }
}

function _with_dummy_out(block) {
  var out = { output: "" };
  var bkup = util.print;
  util.print = function(arg) { out.output += ("" + arg); };
  try {
    return block(out);
  }
  finally {
    util.print = bkup;
  }
}

function _modify(output) {
  return output.replace(/\(.*\/picotest_test\.js/g, '(.../picotest_test.js');
}


(function() {

  var called1 = false, called2 = false;
  var tests = {
    "1+1 should be 2": function() { assert.equal(1+1, 2); called1 = true; },
    "1-1 should be 0": function() { assert.equal(1-1, 0); called2 = true; }
  };
  //
  var ret;
  var output;
  _with_dummy_out(function(out) {
    ret = picotest.run(tests);
    output = out.output;
  });
  should("run(): invoke functions.", function() {
    assert.equal(called1, true);
    assert.equal(called2, true);
  });
  should("run(); return number of errors.", function() {
    assert.equal(ret, 0);
  });
  should("run(): print results.", function() {
    var expected = (""
      + '..\n'
      + '## passed:2, error:0\n'
    );
    assert.equal(output, expected);
  });

})();


(function() {

  var called1 = false, called2 = false, called3 = false;
  var tests = {
    "Ex1": {
      "Ex1-1": {
        "Ex1-1-1": {
          "1+1 should be 2": function() { called1 = true; assert.equal(1+1, 1); }
        },
        "Ex1-1-2": {
          "1-1 should be 0": function() { called2 = true; assert.equal(1-1, 1); },
          "1*1 should be 1": function() { called3 = true; assert.equal(1*1, 1); }
        }
      }
    }
  };
  //
  var ret, output;
  _with_dummy_out(function(out) {
    ret = picotest.run(tests);
    output = out.output;
  });
  should("run(): invoke functions even when error raised.", function() {
    assert.equal(called1, true);
    assert.equal(called2, true);
    assert.equal(called3, true);
  });
  should("run(): return number of errors.", function() {
    assert.equal(ret, 2);
  });
  should("run(): print results with errors.", function() {
    var expected = (
      ""
        + 'EE.\n'
        + '----------------------------------------------------------------------\n'
        + '[Error] Ex1 > Ex1-1 > Ex1-1-1 > 1+1 should be 2\n'
        + 'AssertionError: 2 == 1\n'
        + '    at Object.<anonymous> (.../picotest_test.js:79:66)\n'
        + '----------------------------------------------------------------------\n'
        + '[Error] Ex1 > Ex1-1 > Ex1-1-2 > 1-1 should be 0\n'
        + 'AssertionError: 0 == 1\n'
        + '    at Object.<anonymous> (.../picotest_test.js:82:66)\n'
        + '----------------------------------------------------------------------\n'
        + '## passed:1, error:2\n'
    );
    //console.log(_modify(output));
    assert.equal(_modify(output), expected);
  });

})();


(function() {

  var called1 = false, called2 = false, called3 = false;
  var tests = {
    "Ex1": {
      "Ex1-1": {
        "Ex1-1-1": {
          "1+1 should be 2": function() { called1 = true; assert.equal(1+1, 1); }
        },
        "Ex1-1-2": {
          "1-1 should be 0": function() { called2 = true; assert.equal(1-1, 1); },
          "1*1 should be 1": function() { called3 = true; assert.equal(1*1, 1); }
        }
      }
    }
  };
  //
  var ret, output;
  _with_dummy_out(function(out) {
    ret = picotest.tap(tests);
    output = out.output;
  });
  should("tap(): invoke functions even when error raised.", function() {
    assert.equal(called1, true);
    assert.equal(called2, true);
    assert.equal(called3, true);
  });
  should("tap(): return number of errors.", function() {
    assert.equal(ret, 2);
  });
  should("tap(): print results with errors.", function() {
    var expected = (
      ""
        + '1..3\n'
        + '### Ex1\n'
        + '### Ex1 > Ex1-1\n'
        + '### Ex1 > Ex1-1 > Ex1-1-1\n'
        + 'not ok 1 - 1+1 should be 2\n'
        + '# AssertionError: 2 == 1\n'
        + '#     at Object.<anonymous> (.../picotest_test.js:131:66)\n'
        + '### Ex1 > Ex1-1 > Ex1-1-2\n'
        + 'not ok 2 - 1-1 should be 0\n'
        + '# AssertionError: 0 == 1\n'
        + '#     at Object.<anonymous> (.../picotest_test.js:134:66)\n'
        + 'ok 3 - 1*1 should be 1\n'
    );
    //console.log(_modify(output));
    assert.equal(_modify(output), expected);
  });

})();



(function() {

  var before_called = 0, after_called = 0;
  var tests = {
    "Ex1": {
      "BEFORE": function() { before_called++; },
      "AFTER": function() { after_called++; },
      "ex1": function() { },
      "ex2": function() { },
      "ex3": function() { }
    }
  };
  should("ret(): supports BEFORE() and AFTER().", function() {
    before_called = after_called = 0;
    _with_dummy_out(function(out) { picotest.run(tests); });
    assert.equal(before_called, 3);
    assert.equal(after_called, 3);
  });
  should("tap(): supports BEFORE() and AFTER().", function() {
    before_called = after_called = 0;
    _with_dummy_out(function(out) { picotest.run(tests); });
    assert.equal(before_called, 3);
    assert.equal(after_called, 3);
  });

})();



(function() {

  var called1 = false, called2 = false, called3 = false;
  var tests = {
    "Ex1": {
      "BEFORE": function() { util.print("<BEFORE()>\n"); },
      "AFTER": function() { util.print("<AFTER()>\n"); },
      "example": function() {
        util.print("example()\n");
        notexistvar(); // ReferenceError
      }
    }
  };
  //
  should("ret(): call AFTER() even when error raised.", function() {
    var ret, output;
    _with_dummy_out(function(out) {
      ret = picotest.run(tests);
      output = out.output;
    });
    var expected = (
      ""
        + '<BEFORE()>\n'
        + 'example()\n'
        + 'E<AFTER()>\n'
        + '\n'
        + '----------------------------------------------------------------------\n'
        + '[Error] Ex1 > example\n'
        + 'ReferenceError: notexistvar is not defined\n'
        + '    at Object.<anonymous> (.../picotest_test.js:216:9)\n'
        + '----------------------------------------------------------------------\n'
        + '## passed:0, error:1\n'
      );
    //console.log(output);
    assert.equal(_modify(output), expected);
  });
  //
  should("tap(): call AFTER() even when error raised.", function() {
    var ret, output;
    _with_dummy_out(function(out) {
      ret = picotest.tap(tests);
      output = out.output;
    });
    var expected = (
      ""
        + '1..1\n'
        + '### Ex1\n'
        + '<BEFORE()>\n'
        + 'example()\n'
        + 'not ok 1 - example\n'
        + '# ReferenceError: notexistvar is not defined\n'
        + '#     at Object.<anonymous> (.../picotest_test.js:216:9)\n'
        + '<AFTER()>\n'
      );
    //console.log(output);
    assert.equal(_modify(output), expected);
  });

})();



(function() {

  //
  should("ret(): use 'THIS' as 'this'.", function() {
    var called = [];
    var tests = {
      "Ex1": {
        "BEFORE": function() { this.x = 10; assert.equal(this.z, 30); called.push("BEFORE"); },
        "AFTER":  function() { this.y = 20; assert.equal(this.k, 40); called.push("AFTER"); },
        "THIS": { z: 30 },
        "example": function() {
          assert.equal(this.x, 10);
          assert.equal(this.y, undefined);
          assert.equal(this.z, 30);
          this.k = 40;
          called.push("example");
        }
      }
    };
    _with_dummy_out(function(out) { picotest.run(tests); });
    assert.equal(called.join("|"), "BEFORE|example|AFTER");
  });
  //
  should("tap(): use 'THIS' as 'this'.", function() {
    var called = [];
    var tests = {
      "Ex1": {
        "BEFORE": function() { this.x = 10; assert.equal(this.z, 30); called.push("BEFORE"); },
        "AFTER":  function() { this.y = 20; assert.equal(this.k, 40); called.push("AFTER"); },
        "THIS": { z: 30 },
        "example": function() {
          assert.equal(this.x, 10);
          assert.equal(this.y, undefined);
          assert.equal(this.z, 30);
          this.k = 40;
          called.push("example");
        }
      }
    };
    _with_dummy_out(function(out) { picotest.tap(tests); });
    assert.equal(called.join("|"), "BEFORE|example|AFTER");
  });

})();



(function() {

  //
  should("ret(): share 'THIS' for each tests.", function() {
    var value;
    var tests = {
      "Ex1": {
        "THIS": { val: "" },
        "ex1": function() { this.val += "x"; value = this.val; },
        "ex2": function() { this.val += "x"; value = this.val; },
        "ex3": function() { this.val += "x"; value = this.val; }
      }
    };
    _with_dummy_out(function(out) { picotest.run(tests); });
    assert.equal(value, "xxx");
  });
  //
  should("tap(): share 'THIS' for each tests.", function() {
    var value;
    var tests = {
      "Ex1": {
        "THIS": { val: "" },
        "ex1": function() { this.val += "x"; value = this.val; },
        "ex2": function() { this.val += "x"; value = this.val; },
        "ex3": function() { this.val += "x"; value = this.val; }
      }
    };
    _with_dummy_out(function(out) { picotest.tap(tests); });
    assert.equal(value, "xxx");
  });

})();



(function() {

  //
  should("ret(): call 'THIS' once when it is a function.", function() {
    var called = 0;
    var tests = {
      "Ex1": {
        "THIS": function() { called++; return {x: 10}; },
        "ex1": function() { assert.equal(this.x, 10); },
        "ex2": function() { assert.equal(this.x, 10); },
        "ex3": function() { assert.equal(this.x, 10); }
      }
    };
    var n_errors;
    _with_dummy_out(function(out) { n_errors = picotest.run(tests); });
    assert.equal(called, 1);
    assert.equal(n_errors, 0);
  });
  //
  should("tap(): call 'THIS' once when it is a function.", function() {
    var called = 0;
    var tests = {
      "Ex1": {
        "THIS": function() { called++; return {x: 10}; },
        "ex1": function() { assert.equal(this.x, 10); },
        "ex2": function() { assert.equal(this.x, 10); },
        "ex3": function() { assert.equal(this.x, 10); }
      }
    };
    var n_errors;
    _with_dummy_out(function(out) { n_errors = picotest.tap(tests); });
    assert.equal(called, 1);
    assert.equal(n_errors, 0);
  });
  //

})();



(function() {

  //
  should("count_tests(): counts test functions.", function() {
    var tests = {
      "Ex1": {
        "Ex1-1": {
          "Ex1-1-1": {
            "ex1": function() {},
            "ex2": function() {}
          },
          "Ex1-1-2": {
            "ex3": function() {}
          }
        }
      },
      "ex4": function() {}
    };
    var n = picotest.count_tests(tests);
    assert.equal(n, 4);
  });
  //
  should("count_tests(): excludes BEFORE/AFTER/THIS.", function() {
    var tests = {
      "Ex1": {
        "BEFORE": function() {},
        "AFTER": function() {},
        "Ex1-1": {
          "Ex1-1-1": {
            "BEFORE": function() {},
            "AFTER": function() {},
            "THIS": {},
            "ex1": function() {},
            "ex2": function() {}
          },
          "Ex1-1-2": {
            "ex3": function() {}
          },
          "AFTER": function() {}
        }
      }
    };
    var n = picotest.count_tests(tests);
    assert.equal(n, 3);
  });

})();



(function() {

  //
  should("main(): show help when '-h' specified.", function() {
    var output, ret;
    _with_dummy_out(function(out) {
      ret = picotest.main(['-h', 'foobar']);
      output = out.output;
    });
    var expected = (
      ""
        + 'Usage: picotest.js [options] file_or_directory ...\n'
        + '  -h, --help  : show help\n'
        + '  -v          : print version\n'
        + '  -st         : TAP style\n'
        + '  -sp         : xUnit style (default)\n'
    );
    //console.log(output);
    assert.equal(output, expected);
  });

  //
  should("main(): print version when '-v' specified.", function() {
    var output, ret;
    _with_dummy_out(function(out) {
      ret = picotest.main(['-v', 'foobar']);
      output = out.output;
    });
    var expected = picotest.__version__ + "\n";
    assert.equal(output, expected);
  });

})();



(function() {

  //
  should("withDummyPrint(): capture output of util.print()", function() {
    var output = picotest.withDummyPrint(function() {
      util.print("Haruhi\n");
      util.print("Mikuru\n");
      util.print("Yuki\n");
    });
    assert.equal(output, "Haruhi\nMikuru\nYuki\n");
  });

})();
